#!/usr/bin/env python3
#
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Author: David Bekaert
# Year: 2017
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

import sys
import argparse
import os 

# command line parsing of input file
def cmdLineParse():
    '''
    Command line parser.
    '''
    parser = argparse.ArgumentParser(description='Generate down-sample DEM from wgs84.vrt DEM file')
    parser.add_argument('-i','--input', dest='input_dem_vrt', type=str, required=True, help='Input DEM vrt filename (GDAL supported)')
    parser.add_argument('-r','--res', dest='res_m', type=str, required=True, help='DEM output resolution in m units')
    return parser.parse_args()


# main script
if __name__ == '__main__':
    '''
    Main driver.
    '''
 
    # Parse command line
    inps = cmdLineParse()
    # check if the input file exist
    if not os.path.isfile(inps.input_dem_vrt):
       raise Exception('Input file is not found ....')
    # check if the provided input file is a .vrt file and also get the envi filename
    input_dem_envi, file_extension = os.path.splitext(inps.input_dem_vrt)
    if file_extension != '.vrt':
       raise Exception('Input file is not a vrt file ....')
    # get the file path
    input_path = os.path.dirname(os.path.abspath(inps.input_dem_vrt))
     

    # convert the output resolution from m in degrees 
    # (this is approximate, could use instead exact expression)
    res_degree = float(inps.res_m)/110/1000

    # The ENVI filename of the coarse DEM to be generated
    coarse_dem_envi = os.path.join(input_path, "Coarse_"  + input_dem_envi)

    # Using gdal to down-sample the WGS84 DEM
    cmd = "gdalwarp -t_srs EPSG:4326 -ot Int16 -of ENVI -tr " + str(res_degree) + " " + str(res_degree) + " " + inps.input_dem_vrt + " " + coarse_dem_envi
    os.system(cmd)

    # Generating the ISCE xml and vrt of this coarse DEM
    cmd = "gdal2isce_xml.py -i " + coarse_dem_envi
    os.system(cmd)
