#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Copyright 2012, by the California Institute of Technology. ALL RIGHTS RESERVED.
# United States Government Sponsorship acknowledged. Any commercial use must be
# negotiated with the Office of Technology Transfer at the California Institute of
# Technology.  This software is subject to U.S. export control laws and regulations
# and has been classified as EAR99.  By accepting this software, the user agrees to
# comply with all applicable U.S. export laws and regulations.  User has the
# responsibility to obtain export licenses, or other export authority as may be
# required before exporting such information to foreign countries or providing
# access to foreign persons.
#
# Authors: Walter Szeliga, Eric Gurrola
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



from __future__ import print_function

__all__ = ('createDoppler',)

def useDefault(name=None):
    if name:
        instance = None
    else:
        import isceobj.Doppler.DefaultDopp
        instance = DefaultDopp.DefaultDopp()
        return instance

def useDOPIQ(name=None):
    if name:
        instance = None
    else:
        import mroipac.dopiq.DopIQ
        instance = mroipac.dopiq.DopIQ.DopIQ()
        return instance

def useCalcDop(name=None):
    if name:
        instance = None
    else:
        import isceobj.Doppler.Calc_dop
        instance = isceobj.Doppler.Calc_dop.Calc_dop()
    return instance


def useDoppler(name=None):
    if name:
        instance = None
    else:
        import mroipac.doppler.Doppler
        instance = mroipac.doppler.Doppler.Doppler()
    return instance
    

doppler_facilities = {'USEDOPIQ' : useDOPIQ,
         'USECALCDOP' : useCalcDop,
         'USEDOPPLER' : useDoppler,
         'USEDEFAULT': useDefault}

def getFactoriesInfo():
    """
    Returns a dictionary with information on how to create an object Doppler from its factory
    """
    return  {'Doppler':
                     {'args':
                           {
                            'doppler':{'value':list(doppler_facilities.keys()),'type':'str'}
                            },
                     'factory':'createDoppler'
                     }
              }
    
def createDoppler(doppler=None, name=None):
    if doppler.upper() in doppler_facilities.keys():
        instance = doppler_facilities[doppler.upper()](name)
    else:
        instance = None
        print(
            "Doppler calculation method not recognized. Valid methods: ",
            doppler_facilities.keys())
    return instance

