//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Copyright 2010, by the California Institute of Technology. ALL RIGHTS RESERVED.
// United States Government Sponsorship acknowledged. Any commercial use must be
// negotiated with the Office of Technology Transfer at the California Institute of
// Technology.  This software is subject to U.S. export control laws and regulations
// and has been classified as EAR99.  By accepting this software, the user agrees to
// comply with all applicable U.S. export laws and regulations.  User has the
// responsibility to obtain export licenses, or other export authority as may be
// required before exporting such information to foreign countries or providing
// access to foreign persons.
//
// Author: Giangi Sacco
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



#include "AccessorFactory.h"
#include "CasterFactory.h"
#include "InterleavedFactory.h"
#include "DataCaster.h"
#include "InterleavedBase.h"
#include "DataAccessor.h"
#include "DataAccessorCaster.h"
#include "DataAccessorNoCaster.h"
#include "IQByteToFloatCpxCaster.h"
using namespace std;

DataAccessor *
AccessorFactory::createAccessor(string filename, string accessMode, int size,
    int bands, int width, string interleaved, string caster)
{

  CasterFactory CF;
  InterleavedFactory IF;
  InterleavedBase * interleavedAcc = IF.createInterleaved(interleaved);
  interleavedAcc->init(filename, accessMode, size, bands, width);
  DataCaster * casterD = CF.createCaster(caster);
  return new DataAccessorCaster(interleavedAcc, casterD);
}
DataAccessor *
AccessorFactory::createAccessor(string filename, string accessMode, int size,
    int bands, int width, string interleaved, string caster, float xmi, float xmq, int iqflip)
{

  CasterFactory CF;
  InterleavedFactory IF;
  InterleavedBase * interleavedAcc = IF.createInterleaved(interleaved);
  interleavedAcc->init(filename, accessMode, size, bands, width);
  DataCaster * casterD = CF.createCaster(caster);
  ((IQByteToFloatCpxCaster *) casterD)->setXmi(xmi);
  ((IQByteToFloatCpxCaster *) casterD)->setXmq(xmq);
  ((IQByteToFloatCpxCaster *) casterD)->setIQflip(iqflip);

  return new DataAccessorCaster(interleavedAcc, casterD);

}
DataAccessor *
AccessorFactory::createAccessor(string filename, string accessMode, int size,
    int bands, int width, string interleaved)
{

  InterleavedFactory IF;
  InterleavedBase * interleavedAcc = IF.createInterleaved(interleaved);
  interleavedAcc->init(filename, accessMode, size, bands, width);
  return new DataAccessorNoCaster(interleavedAcc);
}
DataAccessor *
AccessorFactory::createAccessor(void * poly, string interleaved, int width,
    int length, int dataSize)
{
  InterleavedFactory IF;
  InterleavedBase * interleavedAcc = IF.createInterleaved(interleaved);
  interleavedAcc->init(poly);
  interleavedAcc->setLineWidth(width);
  interleavedAcc->setNumberOfLines(length);
  interleavedAcc->setBands(1);
  interleavedAcc->setDataSize(dataSize);

  return new DataAccessorNoCaster(interleavedAcc);
}

void
AccessorFactory::finalize(DataAccessor * dataAccessor)
{
  dataAccessor->finalize();
}
