#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Copyright 2010, by the California Institute of Technology. ALL RIGHTS RESERVED.
# United States Government Sponsorship acknowledged. Any commercial use must be
# negotiated with the Office of Technology Transfer at the California Institute of
# Technology.  This software is subject to U.S. export control laws and regulations
# and has been classified as EAR99.  By accepting this software, the user agrees to
# comply with all applicable U.S. export laws and regulations.  User has the
# responsibility to obtain export licenses, or other export authority as may be
# required before exporting such information to foreign countries or providing
# access to foreign persons.
#
# Author: Giangi Sacco
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



from .formslc import *
from .resamp import *
from .resamp_image import *
from .resamp_amps import *
from .resamp_only import *
from .resamp_slc import *
from .topo import *
from .correct import createCorrect, contextCorrect
from .mocompTSX import *
from .estamb import *

#ing added sensor argument to turn it into a real factory, allowing other type
# of formSLC and moved instantiation here
def createFormSLC(sensor=None, name=''):
    if sensor is None or 'uavsar' in sensor.lower():
        from .formslc.Formslc import Formslc as cls
        return cls(name=name)
    elif str(sensor).lower() in ['terrasarx','cosmo_skymed_slc','radarsat2','sentinel1a','tandemx','kompsat5','risat1_slc','alos2','ers_slc','alos_slc','envisat_slc', 'ers_envisat_slc']:
        from .mocompTSX.MocompTSX import MocompTSX as cls
    else:
        raise ValueError("Unrecognized Sensor: %s" % str(sensor))
    return cls()

def getFactoriesInfo():
    """
    Returns a dictionary with information on how to create an object Sensor from its factory
    """
    return  {'FormSLC':
                     {'args':
                           {
                            'sensor':{'value':['None','uavsar','terrasarx','cosmo_skymed_slc','radarsat2','sentinel1a','tandemx',
                                               'kompsat5','risat1_slc','alos2','ers_slc','alos_slc','envisat_slc'],
                                      'type':'str','optional':True,'default':None}
                            },
                     'factory':'createFormSLC'
                     }
              }


