#!/usr/bin/env python3 

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Copyright 2014, by the California Institute of Technology. ALL RIGHTS RESERVED.
# United States Government Sponsorship acknowledged. Any commercial use must be
# negotiated with the Office of Technology Transfer at the California Institute of
# Technology.  This software is subject to U.S. export control laws and regulations
# and has been classified as EAR99.  By accepting this software, the user agrees to
# comply with all applicable U.S. export laws and regulations.  User has the
# responsibility to obtain export licenses, or other export authority as may be
# required before exporting such information to foreign countries or providing
# access to foreign persons.
#
# Author: Piyush Agram
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



import isce
import datetime
import isceobj
from isceobj.Planet.Planet import Planet
from isceobj.Orbit.Orbit import StateVector, Orbit
from isceobj.Orbit.OrbitExtender import OrbitExtender
from isceobj.Planet.AstronomicalHandbook import Const
from iscesys.Component.Component import Component
from iscesys.DateTimeUtil.DateTimeUtil import DateTimeUtil as DTUtil
import os
import glob
import numpy as np

def createParser():
    import argparse

    parser = argparse.ArgumentParser( description = 'Sentinel parser' )

    parser.add_argument('-d', '--dirname', dest='dirname', type=str,
            default=None, help='SAFE format directory. (Recommended)')

    parser.add_argument('-o', '--outdir', dest='outdir', type=str,
            required=True, help='Output SLC prefix.')

    parser.add_argument('-p', '--orbit', dest='orbit', type=str,
            default=None, help='Precise orbit file, Use of orbitdir preferred')

    parser.add_argument('-a', '--aux', dest='auxprod', type=str,
            default=None, help='Auxiliary product with antenna gains, Use of auxdir preferred')

    parser.add_argument('--orbitdir', dest='orbitdir', type=str,
            default=None, help = 'Directory with all the orbits')

    parser.add_argument('--auxdir', dest='auxdir', type=str,
            default=None, help = 'Directory with all the aux products')

    parser.add_argument('--pol', dest='polid', type=str,
            default='vv', help = 'Polarization of interest. Default: vv')

    parser.add_argument('-b', '--bbox', dest='bbox', type=str,
            default=None, help='Lat/Lon Bounding SNWE')

    parser.add_argument('-s', '--swaths', dest='swaths', type=str,
            default=None, help='list pf swaths')
    return parser


def main(iargs=None):
    from isceobj.Sensor.TOPS.Sentinel1 import Sentinel1

    
    inps = cmdLineParse(iargs)

    if inps.swaths is None:
       inps.swaths = [1,2,3]
    else:
       inps.swaths = [int(i) for i in inps.swaths.split()]

    for swath in inps.swaths:

       obj = Sentinel1()
       obj.configure()
       obj.safe = inps.dirname.split()
       obj.swathNumber = swath
       obj.output = os.path.join(inps.outdir, 'IW{0}'.format(swath))
       obj.orbitFile = inps.orbit
       obj.auxFile = inps.auxprod
       obj.orbitDir = inps.orbitdir
       obj.auxDir = inps.auxdir
       obj.polarization = inps.polid
       if inps.bbox is not None:
          obj.regionOfInterest = [float(x) for x in inps.bbox.split()]
       try:
          obj.parse()
          obj.extractImage(virtual=True) 
       except Exception as e:
          print (e)

def cmdLineParse(iargs=None):
    '''
    Command Line Parser.
    '''
    parser = createParser()
    inps = parser.parse_args(args=iargs)
    return inps

if __name__ == '__main__':
     
    main()



