#!/usr/bin/env python3
#

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Copyright 2016, by the California Institute of Technology. ALL RIGHTS RESERVED.
# United States Government Sponsorship acknowledged. Any commercial use must be
# negotiated with the Office of Technology Transfer at the California Institute of
# Technology.  This software is subject to U.S. export control laws and regulations
# and has been classified as EAR99.  By accepting this software, the user agrees to
# comply with all applicable U.S. export laws and regulations.  User has the
# responsibility to obtain export licenses, or other export authority as may be
# required before exporting such information to foreign countries or providing
# access to foreign persons.
#
# Author: David Bekaert
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~




import os
import glob
import sys
import isce
import isceobj
import argparse
from contrib.UnwrapComp.unwrapComponents import UnwrapComponents


def createParser():
    '''
    Create command line parser.
    '''

    parser = argparse.ArgumentParser(description='Unwrap interferogram using snaphu')
    parser.add_argument('-i', '--ifg', dest='intfile', type=str, required=True,
            help='Input interferogram')
    parser.add_argument('-c', '--coh', dest='cohfile', type=str, required=True,
            help='Coherence file')
    parser.add_argument('-u', '--unwprefix', dest='unwprefix', type=str, required=False,
            help='Output unwrapped file prefix')
    parser.add_argument('--nomcf', action='store_true', default=False,
            help='Run full snaphu and not in MCF mode, default = False')
    parser.add_argument('-a','--alks', dest='azlooks', type=int, default=1,
            help='Number of azimuth looks, default =1')
    parser.add_argument('-r', '--rlks', dest='rglooks', type=int, default=1,
            help='Number of range looks, default =1')
    parser.add_argument('-d', '--defomax', dest='defomax', type=float, default=2.0,
            help='Max cycles of deformation, default =2')    
    parser.add_argument('-m', '--method', dest='method', type=str, default='snaphu2stage',
            help='unwrapping method (snaphu, snaphu2stage= default, icu)')
    parser.add_argument('--overwrite', action='store_true', default=True,
            help='Overwrite file on re-run, default = True ===>>>> 2B implemented')
    return parser


def cmdLineParse(iargs=None):
    '''
    Command line parser.
    '''

    parser = createParser()
    return parser.parse_args(args = iargs)
    
 
def main(iargs=None):
    '''
    The main driver.
    ''' 

    inps = cmdLineParse(iargs)
    print(inps.method)
    if (inps.method != 'icu') and (inps.method != 'snaphu') and (inps.method != 'snaphu2stage'):
        raise Exception("Unwrapping method needs to be either icu, snaphu or snaphu2stage")



    # passign arguments
    if inps.nomcf: 
         nomcf_str =  " --nomcf "
    else:
         nomcf_str =  " "
    if inps.unwprefix:
        unwprefix_str = " - u " + inps.unwprefix
    else:
        unwprefix_str = " "



    #Get current directory
    currdir = os.getcwd()

    ##### Loop over the different intergerograms 
    for dirf in glob.glob(os.path.join(currdir, '2*',inps.intfile)):
        vals = dirf.split(os.path.sep)
        date = vals[-2]
        print(date)
        os.chdir(date)
        cmd = "step_unwrap_isce.py -i " + inps.intfile + " -c " + inps.cohfile + " -a " + str(inps.azlooks) + " -r " + str(inps.rglooks) + nomcf_str + " -m " + inps.method + unwprefix_str
        print(cmd)
        os.system(cmd)
        os.chdir('../.')

        continue

if __name__ == '__main__':

    main()
